/** @file
 * @brief Implementation of intuitive tristate logic
 *
 * (c) Copyright 2011 by Intra2net AG
 * Written by Erich Willems and Thomas Jarosch
 *
 * info@intra2net.com
 */

#include <tribool.hpp>

namespace I2n
{

/**
    * @brief Constructor
    * Default to Undef state
    *
    **/
Tribool::Tribool()
    : Value(Undef)
{
}

/**
 * @brief Constructor
 *
 * @param v Initial state
 **/
Tribool::Tribool(const State &v)
    : Value(v)
{
}

/**
 * @brief Copy constructor
 *
 * @param src Class to copy the value from
 **/
Tribool::Tribool(const Tribool &src)
    : Value(src.Value)
{
}

/**
 * @brief Assignment operator
 *
 * @param v New state
 * @return :Tribool& Pointer to this class
 **/
Tribool& Tribool::operator= (const State &v)
{
    Value=v;
    return *this;
}

/**
 * @brief Assignment operator
 *
 * @param src Class to copy the state from
 * @return :Tribool& Pointer to this class
 **/
Tribool& Tribool::operator= (const Tribool& src)
{
    Value=src.Value;
    return *this;
} //lint !e1529

/**
 * @brief Element comparator
 *
 * @param x Class to compare against
 * @return bool True if the are in 100% the same state, false otherwise
 * Will return true if both elements are "Undef".
 **/
bool Tribool::operator==(const Tribool& x) const
{
    return Value==x.Value;
}

/**
 * @brief Element comparator, NOT version
 *
 * @param x Class to compare against
 * @return bool True if they are not in the same state, false otherwise
 **/
bool Tribool::operator!=(const Tribool& x) const
{
    return Value!=x.Value;
}

/**
 * @brief Logical NOT operator - operating in tristate space
 *
 * @return :Tribool Tribool state as the result
 **/
Tribool Tribool::operator !() const
{
    switch (Value)
    {
        case True:
            return False;
        case False:
            return True;
    } //lint !e787

    return Undef;
}

/**
 * @brief Logical OR - operating in tristate space
 *
 * @param x Object to compare against
 * @return :Tribool Tribool state as result
 **/
Tribool Tribool::operator ||(const Tribool& x) const                            //lint !e1753
{
    if ((True==Value)  || (True==x.Value))
        return True;

    if ((False==Value) && (False==x.Value))
        return False;

    return Undef;
}

/**
 * @brief Logical AND operator - operating in tristate space
 *
 * @param x Object to compare against
 * @return :Tribool Tristate state as result
 **/
Tribool Tribool::operator &&(const Tribool& x) const                            //lint !e1753
{
    if ((True==Value)  && (True==x.Value))
        return True;

    if ((False==Value) || (False==x.Value))
        return False;

    return Undef;
}

/**
 * @brief Constructor with bool conversion
 *
 * @param v Initial value in bistate space: true or false
 **/
Tribool::Tribool(const bool &v)
{
    if (v == true)
        Value = True;
    else
        Value = False;
}

/**
 * @brief Assignment operator
 *
 * @param v Value in bistate space: true or false
 * @return :Tribool& Pointer to this class
 **/
Tribool& Tribool::operator=(const bool &v)
{
    if (v == true)
        Value = True;
    else
        Value = False;

    return *this;
}

/**
 * @brief Element comparision for easier access in bistate space
 *
 * This is lossy!
 *
 * @param v Boolean value to compare against
 * @return bool True if it's exact the bool value, false otherwise.
 * Undef state will return false.
 **/
bool Tribool::is_exact(const bool &v) const
{
    if (Value==Undef)
        return false;

    if (v == true && Value == True)
        return true;
    else if(v == false && Value == False)
        return true;

    return false;
}

}
